@@ -1029,8 +1029,14 @@ vu_read_retry (int * errn, int fd, char * buf, size_t count)
     {
       ssize_t amt;
       amt = vu_read (errn, fd, buf, count);
-      if ((amt < 0) && (*errn != EAGAIN) && (*errn != EINTR))
-	return amt;
+      if (amt < 0)
+        {
+          if ((*errn == EAGAIN) || (*errn == EINTR))
+            continue;
+          else
+            return amt;
+        }
+      
       amt_read += amt;
       if (!amt || (amt == count))
 	return amt_read;
@@ -1237,12 +1243,20 @@ vu_write_retry (int * errn, int fd, char * buf, size_t count)
       ssize_t amt;
       
       amt = vu_write (errn, fd, buf, count);
-      if ((amt < 0) && (*errn != EAGAIN) && (*errn != EINTR))
-	return amt;
-      if (amt == count)
+      if (amt < 0)
+        {
+          if ((*errn == EAGAIN) || (*errn == EINTR))
+            continue;
+          else
+            return amt;
+        }
+      else if (amt == count)
 	return orig_count;
-      count -= amt;
-      buf += amt;
+      else
+        {
+          count -= amt;
+          buf += amt;
+        }
     }
 }
 
