@@ -22,6 +22,7 @@
 #include "hackerlab/fs/file-names.h"
 #include "hackerlab/fs/cwd.h"
 #include "hackerlab/vu/safe.h"
+#include "hackerlab/char/pika-escaping-utils.h"
 #include "tla/libfsutils/link-target.h"
 #include "tla/libfsutils/read-line.h"
 #include "tla/libfsutils/tmp-files.h"
@@ -149,7 +150,8 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
                       enum arch_id_tagging_method method,
                       enum arch_inventory_category untagged_source_category,
                       int reverse, int forward,
-                      int use_diff3, assoc_table older_table, assoc_table yours_table)
+                      int use_diff3, assoc_table older_table, assoc_table yours_table,
+                      int escape_classes)
 {
   int x;
   int here_fd;
@@ -224,7 +226,8 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
    * Inventory the target tree.
    */
   mem_set0 ((t_uchar *)&inventory, sizeof (inventory));
-  arch_changeset_inventory (&inventory, tree_root, target, method, untagged_source_category);
+  arch_changeset_inventory (&inventory, tree_root, target, method,
+                            untagged_source_category, escape_classes);
 
   mem_set0 ((t_uchar *)&inventory_by_name, sizeof (inventory_by_name));
 
@@ -273,6 +276,7 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
       t_uchar * target_path = 0;
       t_uchar * dest_path = 0;
       t_uchar * dest_dir = 0;
+      t_uchar * escaped_tmp = 0;
 
       target_loc = r->removed_files[x][0];
       target_id = r->removed_files[x][1];
@@ -280,13 +284,16 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
       dest_path = file_name_in_vicinity (0, tmp_removed_files_root, target_loc);
       dest_dir = file_name_directory_file (0, dest_path);
 
-      invoke_apply_changeset_callback (r, "D   %s\n", no_dot (target_loc));
+      escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, target_loc);
+      invoke_apply_changeset_callback (r, "D   %s\n", no_dot (escaped_tmp));
+
       ensure_directory_exists (dest_dir);
       safe_rename (target_path, dest_path);
 
       assoc_del (running.file_id_of, target_loc);
       assoc_del (running.file_loc_of, target_id);
 
+      lim_free (0, escaped_tmp);
       lim_free (0, target_path);
       lim_free (0, dest_path);
       lim_free (0, dest_dir);
@@ -304,9 +311,15 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
   r->missing_renamed_files = rel_join (1, rel_join_output (1,0, 1,1, 1,2, -1), 2, 1, renamed_files_index, inventory.files);
   for (x = 0; x < rel_n_records (r->missing_renamed_files); ++x)
     {
+      t_uchar * escaped_tmp0;
+      t_uchar * escaped_tmp1;
+      escaped_tmp0 = pika_save_escape_iso8859_1 (0, 0, escape_classes, r->missing_renamed_files[x][0]);
+      escaped_tmp1 = pika_save_escape_iso8859_1 (0, 0, escape_classes, r->missing_renamed_files[x][1]);
       invoke_apply_changeset_callback (r, "?r  %s\n     => %s\n",
-                                       no_dot (r->missing_renamed_files[x][0]),
-                                       no_dot (r->missing_renamed_files[x][1]));
+                                       no_dot (escaped_tmp0),
+                                       no_dot (escaped_tmp1));
+      lim_free (0, escaped_tmp1);
+      lim_free (0, escaped_tmp0);
     }
 
   rel_sort_table_by_field (0, r->missing_removed_files, 0);
@@ -354,9 +367,15 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
   rel_sort_table_by_field (0, r->missing_renamed_dirs, 0);
   for (x = 0; x < rel_n_records (r->missing_renamed_dirs); ++x)
     {
+      t_uchar * escaped_tmp0;
+      t_uchar * escaped_tmp1;
+      escaped_tmp0 = pika_save_escape_iso8859_1 (0, 0, escape_classes, r->missing_renamed_dirs[x][0]);
+      escaped_tmp1 = pika_save_escape_iso8859_1 (0, 0, escape_classes, r->missing_renamed_dirs[x][1]);
       invoke_apply_changeset_callback (r, "?r/ %s\n     => %s\n",
-                                       no_dot (r->missing_renamed_dirs[x][0]),
-                                       no_dot (r->missing_renamed_dirs[x][1]));
+                                       no_dot (escaped_tmp0),
+                                       no_dot (escaped_tmp1));
+      lim_free (0, escaped_tmp1);
+      lim_free (0, escaped_tmp0);
     }
 
   removed_dirs_index = rel_copy_table (changeset.removed_dirs);
@@ -412,12 +431,16 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
           }
         else
           {
+            t_uchar * escaped_tmp;
             target_loc = present_removed_dirs_index[rem_pos][0];
             id = present_removed_dirs_index[rem_pos][1];
             dest_record = &present_removed_dirs_index[rem_pos];
             ++rem_pos;
 
-            invoke_apply_changeset_callback (r, "D/  %s\n", no_dot (target_loc));
+            escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, target_loc);
+            invoke_apply_changeset_callback (r, "D/  %s\n", no_dot (escaped_tmp));
+            lim_free (0, escaped_tmp);
+
             rel_add_records (&r->removed_dirs, rel_make_record (target_loc, id, 0), 0);
           }
         tmp_name = set_aside_shuffled_dirs (&file_set_aside_with_dir_id, &dir_set_aside_with_dir_id, target_loc, id, seq, tmp_shuffled_dirs_root, &running, target, &inventory);
@@ -543,6 +566,7 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
               int ign;
               t_uchar * perms_str;
               unsigned long perms;
+              t_uchar * escaped_tmp = 0;
 
               perms_str = assoc_ref (new_dir_perms, mod_loc);
               if (cvt_octal_to_ulong (&ign, &perms, perms_str, str_length (perms_str)))
@@ -551,40 +575,51 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
                   trust_umask = 1;
                 }
 
+              escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, install_loc);
+              
               if (!deferred_conflict (&deferred_conflicts, removed_patch_conflict_files_path, target, install_loc, 0))
                 {
-                  invoke_apply_changeset_callback (r, "A/  %s\n", no_dot (install_loc));
+                  invoke_apply_changeset_callback (r, "A/  %s\n", no_dot (escaped_tmp));
                   rel_add_records (&r->new_dirs, rel_make_record (install_loc, id, 0), 0);
                 }
               else
                 {
-                  invoke_apply_changeset_callback (r, "CA/ %s\n", no_dot (install_loc));
+                  invoke_apply_changeset_callback (r, "CA/ %s\n", no_dot (escaped_tmp));
                   rel_add_records (&r->conflict_dirs, rel_make_record (install_loc, id, 0), 0);
                 }
 
               safe_mkdir (install_name, perms);
               if (!trust_umask)
                 safe_chmod (install_name, perms);
+
+              lim_free (0, escaped_tmp);
             }
         }
       else
         {
           t_uchar * oldtgtloc;
+          t_uchar * escaped_tmp0 = 0;
+          t_uchar * escaped_tmp1 = 0;
 
           oldtgtloc = install_dirs_plan[x][3];
 
+          escaped_tmp0 = pika_save_escape_iso8859_1 (0, 0, escape_classes, oldtgtloc);
+          escaped_tmp1 = pika_save_escape_iso8859_1 (0, 0, escape_classes, install_loc);
+
           if (!deferred_conflict (&deferred_conflicts, removed_patch_conflict_files_path, target, install_loc, 0))
             {
-              invoke_apply_changeset_callback (r, "/>  %s\t%s\n", oldtgtloc, no_dot (install_loc));
+              invoke_apply_changeset_callback (r, "/>  %s\t%s\n", escaped_tmp0, no_dot (escaped_tmp1));
               rel_add_records (&r->renamed_dirs, rel_make_record (oldtgtloc, install_loc, id, 0), 0);
             }
           else
             {
-              invoke_apply_changeset_callback (r, "C/> %s\t%s\n", oldtgtloc, no_dot (install_loc));
+              invoke_apply_changeset_callback (r, "C/> %s\t%s\n", escaped_tmp0, no_dot (escaped_tmp1));
               rel_add_records (&r->conflict_dirs, rel_make_record (install_loc, id, 0), 0);
             }
 
           safe_rename (take_from, install_name);
+          lim_free (0, escaped_tmp1);
+          lim_free (0, escaped_tmp0);
         }
 
       lim_free (0, target_has_dir);
@@ -616,6 +651,9 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
 
       t_uchar * target_has_path = 0;
 
+      t_uchar * escaped_tmp0 = 0;
+      t_uchar * escaped_tmp1 = 0;
+
       old_target_loc = present_renamed_files_index[x][0];
       mod_loc = present_renamed_files_index[x][2];
       id = present_renamed_files_index[x][3];
@@ -634,19 +672,24 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
                                               removed_patch_conflict_files_path,
                                               target_has_path, install_dir, target);
 
+      escaped_tmp0 = pika_save_escape_iso8859_1 (0, 0, escape_classes, old_target_loc);
+      escaped_tmp1 = pika_save_escape_iso8859_1 (0, 0, escape_classes, install_loc);
+
       if (!deferred_conflict (&deferred_conflicts, removed_patch_conflict_files_path, target, install_loc, 0))
         {
-          invoke_apply_changeset_callback (r, "=>  %s\t%s\n", old_target_loc, no_dot (install_loc));
+          invoke_apply_changeset_callback (r, "=>  %s\t%s\n", escaped_tmp0, no_dot (escaped_tmp1));
           rel_add_records (&r->renamed_files, rel_make_record (old_target_loc, install_loc, id, 0), 0);
         }
       else
         {
-          invoke_apply_changeset_callback (r, "C=> %s\t%s\n", old_target_loc, no_dot (install_loc));
+          invoke_apply_changeset_callback (r, "C=> %s\t%s\n", escaped_tmp0, no_dot (escaped_tmp1));
           rel_add_records (&r->conflict_files, rel_make_record (install_loc, id, 0), 0);
         }
 
       safe_rename (take_from, install_name);
 
+      lim_free (0, escaped_tmp1);
+      lim_free (0, escaped_tmp0);
       lim_free (0, take_from);
       lim_free (0, target_has_dir);
       lim_free (0, install_dir);
@@ -672,6 +715,8 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
 
       t_uchar * target_has_path = 0;
 
+      t_uchar * escaped_tmp = 0;
+
       mod_loc = added_files_and_symlinks[x][0];
       id = added_files_and_symlinks[x][1];
       take_from = added_files_and_symlinks[x][2];
@@ -688,20 +733,23 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
                                               removed_patch_conflict_files_path,
                                               target_has_path, install_dir, target);
 
+      escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, install_loc);
+      
       if (!deferred_conflict (&deferred_conflicts, removed_patch_conflict_files_path, target, install_loc, take_from))
         {
-          invoke_apply_changeset_callback (r, "A   %s\n", no_dot (install_loc));
+          invoke_apply_changeset_callback (r, "A   %s\n", no_dot (escaped_tmp));
           rel_add_records (&r->new_files, rel_make_record (install_loc, id, 0), 0);
         }
       else
         {
-          invoke_apply_changeset_callback (r, "CA  %s\n", no_dot (install_loc));
+          invoke_apply_changeset_callback (r, "CA  %s\n", no_dot (escaped_tmp));
           rel_add_records (&r->conflict_files, rel_make_record (install_loc, id, 0), 0);
         }
 
       copy_file_or_symlink (take_from, install_name);
       copy_permissions (take_from, install_name);
 
+      lim_free (0, escaped_tmp);
       lim_free (0, target_has_dir);
       lim_free (0, install_dir);
       lim_free (0, install_name);
@@ -723,6 +771,7 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
       t_uchar * target_loc;
       t_uchar * patch_path = 0;
       t_uchar * target_path = 0;
+      t_uchar * escaped_tmp = 0;
       struct stat target_stat;
 
       patch_path = str_alloc_cat (0, changeset.patched_regular_files[x][2], ".patch");
@@ -732,9 +781,11 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
 
       if (!target_loc)
         {
-          invoke_apply_changeset_callback (r, "?M  %s\n", no_dot (mod_loc));
+          escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, mod_loc);
+          invoke_apply_changeset_callback (r, "?M  %s\n", no_dot (escaped_tmp));
           rel_add_records (&r->missing_file_for_patch, rel_make_record (mod_loc, id, 0), 0);
           save_patch_for_missing_file (missing_patch_dir, patch_path, mod_loc);
+          lim_free (0, escaped_tmp);
           continue;
         }
 
@@ -755,8 +806,10 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
           copy_file (patch_path, rej_name);
           copy_permissions (patch_path, rej_name);
 
-          invoke_apply_changeset_callback (r, "C-> %s\n", no_dot (target_loc));
+          escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, target_loc);
+          invoke_apply_changeset_callback (r, "C-> %s\n", no_dot (escaped_tmp));
           rel_add_records (&r->conflict_files, rel_make_record (target_loc, id, 0), 0);
+          lim_free (0, escaped_tmp);
         }
       else if (arch_id_indicates_changelog (id))
         {
@@ -776,7 +829,6 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
               t_uchar * original_inode_basename = 0;
               t_uchar * original_inode_tmpname = 0;
               int patch_stat = -1;
-              int used_diff3 = 0;
 
               target_dir = file_name_directory_file (0, target_path);
               basename = file_name_tail (0, target_path);
@@ -812,8 +864,6 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
                     patch_stat = run_diff3 (basename, original_inode_tmpname, older_path, yours_path);
                   else
                     patch_stat = run_diff3 (basename, original_inode_tmpname, yours_path, older_path);
-
-                  used_diff3 = 1;
                 }
 
               if (patch_stat == 0)
@@ -821,9 +871,11 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
                   copy_permissions (original_inode_tmpname, basename);
                   safe_unlink (original_inode_tmpname);
 
+                  escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, target_loc);
                   invoke_apply_changeset_callback (r, "M   %s\n",
-                                                   no_dot (target_loc));
+                                                   no_dot (escaped_tmp));
                   rel_add_records (&r->modified_files, rel_make_record (target_loc, id, 0), 0);
+                  lim_free (0, escaped_tmp);
                 }
               else if (patch_stat == 1)
                 {
@@ -833,10 +885,12 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
                   orig_name = str_alloc_cat (0, basename, ".orig");
                   safe_rename (original_inode_tmpname, orig_name);
 
+                  escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, target_loc);
                   invoke_apply_changeset_callback (r, "C   %s\n",
-                                                   no_dot (target_loc));
+                                                   no_dot (escaped_tmp));
                   rel_add_records (&r->conflict_files, rel_make_record (target_loc, id, 0), 0);
 
+                  lim_free (0, escaped_tmp);
                   lim_free (0, orig_name);
                 }
               else
@@ -885,6 +939,7 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
       t_uchar * orig_patch_path = 0;
       t_uchar * mod_patch_path = 0;
       t_uchar * target_path = 0;
+      t_uchar * escaped_tmp = 0;
       struct stat target_stat;
 
       if (!reverse)
@@ -904,10 +959,12 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
 
       if (!target_loc)
         {
-          invoke_apply_changeset_callback (r, "?M  %s\n", no_dot (mod_loc));
+          escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, mod_loc);
+          invoke_apply_changeset_callback (r, "?M  %s\n", no_dot (escaped_tmp));
           rel_add_records (&r->missing_file_for_patch, rel_make_record (mod_loc, id, 0), 0);
           save_patch_for_missing_file (missing_patch_dir, orig_patch_path, mod_loc);
           save_patch_for_missing_file (missing_patch_dir, mod_patch_path, mod_loc);
+          lim_free (0, escaped_tmp);
           continue;
         }
 
@@ -942,9 +999,11 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
           safe_close (in_fd);
           safe_printfmt (out_fd, "\n");
 
-          invoke_apply_changeset_callback (r, "C-> %s\n", no_dot (target_loc));
+          escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, target_loc);
+          invoke_apply_changeset_callback (r, "C-> %s\n", no_dot (escaped_tmp));
           rel_add_records (&r->conflict_files, rel_make_record (target_loc, id, 0), 0);
 
+          lim_free (0, escaped_tmp);
           lim_free (0, orig_name);
           lim_free (0, rej_name);
         }
@@ -1001,9 +1060,11 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
 
           if (patch_stat == 0)
             {
+              escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, target_loc);
               invoke_apply_changeset_callback (r, "M-> %s\n",
-                                               no_dot (target_loc));
+                                               no_dot (escaped_tmp));
               rel_add_records (&r->modified_files, rel_make_record (target_loc, id, 0), 0);
+              lim_free (0, escaped_tmp);
             }
           else
             goto symlink_conflict;
@@ -1031,6 +1092,7 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
       t_uchar * orig_patch_path = 0;
       t_uchar * mod_patch_path = 0;
       t_uchar * target_path = 0;
+      t_uchar * escaped_tmp = 0;
       struct stat target_stat;
 
       if (!reverse)
@@ -1050,10 +1112,12 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
 
       if (!target_loc)
         {
-          invoke_apply_changeset_callback (r, "?M  %s\n", no_dot (mod_loc));
+          escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, mod_loc);
+          invoke_apply_changeset_callback (r, "?M  %s\n", no_dot (escaped_tmp));
           rel_add_records (&r->missing_file_for_patch, rel_make_record (mod_loc, id, 0), 0);
           save_patch_for_missing_file (missing_patch_dir, orig_patch_path, mod_loc);
           save_patch_for_missing_file (missing_patch_dir, mod_patch_path, mod_loc);
+          lim_free (0, escaped_tmp);
           continue;
         }
 
@@ -1076,9 +1140,11 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
           copy_file (mod_patch_path, rej_name);
           copy_permissions (mod_patch_path, rej_name);
 
-          invoke_apply_changeset_callback (r, "Cb  %s\n", no_dot (target_loc));
+          escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, target_loc);
+          invoke_apply_changeset_callback (r, "Cb  %s\n", no_dot (escaped_tmp));
           rel_add_records (&r->conflict_files, rel_make_record (target_loc, id, 0), 0);
 
+          lim_free (0, escaped_tmp);
           lim_free (0, orig_name);
           lim_free (0, rej_name);
         }
@@ -1117,9 +1183,11 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
 
           if (patch_stat == 0)
             {
+              escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, target_loc);
               invoke_apply_changeset_callback (r, "Mb  %s\n",
-                                               no_dot (target_loc));
+                                               no_dot (escaped_tmp));
               rel_add_records (&r->modified_files, rel_make_record (target_loc, id, 0), 0);
+              lim_free (0, escaped_tmp);
             }
           else
             goto binary_conflict;
@@ -1143,6 +1211,7 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
       t_uchar * orig_patch_path = 0;
       t_uchar * mod_patch_path = 0;
       t_uchar * target_path = 0;
+      t_uchar * escaped_tmp = 0;
       struct stat target_stat;
 
       if (!reverse)
@@ -1162,10 +1231,12 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
 
       if (!target_loc)
         {
-          invoke_apply_changeset_callback (r, "?M  %s\n", no_dot (mod_loc));
+          escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, mod_loc);
+          invoke_apply_changeset_callback (r, "?M  %s\n", no_dot (escaped_tmp));
           rel_add_records (&r->missing_file_for_patch, rel_make_record (mod_loc, id, 0), 0);
           save_patch_for_missing_file (missing_patch_dir, orig_patch_path, mod_loc);
           save_patch_for_missing_file (missing_patch_dir, mod_patch_path, mod_loc);
+          lim_free (0, escaped_tmp);
           continue;
         }
 
@@ -1188,9 +1259,11 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
           copy_file (mod_patch_path, rej_name);
           copy_permissions (mod_patch_path, rej_name);
 
-          invoke_apply_changeset_callback (r, "Cch %s\n", no_dot (target_loc));
+          escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, target_loc);
+          invoke_apply_changeset_callback (r, "Cch %s\n", no_dot (escaped_tmp));
           rel_add_records (&r->conflict_files, rel_make_record (target_loc, id, 0), 0);
 
+          lim_free (0, escaped_tmp);
           lim_free (0, orig_name);
           lim_free (0, rej_name);
         }
@@ -1234,9 +1307,11 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
 
           if (patch_stat == 0)
             {
-              invoke_apply_changeset_callback (r, "ch %s\n",
-                                               no_dot (target_loc));
+              escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, target_loc);
+              invoke_apply_changeset_callback (r, "ch  %s\n",
+                                               no_dot (escaped_tmp));
               rel_add_records (&r->modified_files, rel_make_record (target_loc, id, 0), 0);
+              lim_free (0, escaped_tmp);
             }
           else
             goto symlink_to_file_conflict;
@@ -1262,6 +1337,7 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
       t_uchar * orig_patch_path = 0;
       t_uchar * mod_patch_path = 0;
       t_uchar * target_path = 0;
+      t_uchar * escaped_tmp = 0;
       struct stat target_stat;
 
       if (!reverse)
@@ -1281,10 +1357,12 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
 
       if (!target_loc)
         {
-          invoke_apply_changeset_callback (r, "?M  %s\n", no_dot (mod_loc));
+          escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, mod_loc);
+          invoke_apply_changeset_callback (r, "?M  %s\n", no_dot (escaped_tmp));
           rel_add_records (&r->missing_file_for_patch, rel_make_record (mod_loc, id, 0), 0);
           save_patch_for_missing_file (missing_patch_dir, orig_patch_path, mod_loc);
           save_patch_for_missing_file (missing_patch_dir, mod_patch_path, mod_loc);
+          lim_free (0, escaped_tmp);
           continue;
         }
 
@@ -1314,9 +1392,11 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
           safe_close (in_fd);
           safe_close (out_fd);
 
-          invoke_apply_changeset_callback (r, "Cch %s\n", no_dot (target_loc));
+          escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, target_loc);
+          invoke_apply_changeset_callback (r, "Cch %s\n", no_dot (escaped_tmp));
           rel_add_records (&r->conflict_files, rel_make_record (target_loc, id, 0), 0);
 
+          lim_free (0, escaped_tmp);
           lim_free (0, orig_name);
           lim_free (0, rej_name);
         }
@@ -1358,9 +1438,11 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
 
           if (patch_stat == 0)
             {
-              invoke_apply_changeset_callback (r, "ch %s\n",
-                                               no_dot (target_loc));
+              escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, target_loc);
+              invoke_apply_changeset_callback (r, "ch  %s\n",
+                                               no_dot (escaped_tmp));
               rel_add_records (&r->modified_files, rel_make_record (target_loc, id, 0), 0);
+              lim_free (0, escaped_tmp);
             }
           else
             goto file_to_symlink_conflict;
@@ -1388,6 +1470,7 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
       t_uchar * target_path = 0;
       t_uchar * target_spew_loc = 0;
       t_uchar * target_spew_path = 0;
+      t_uchar * escaped_tmp = 0;
       struct stat target_stat;
 
       if (!reverse)
@@ -1407,10 +1490,12 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
 
       if (!target_loc)
         {
-          invoke_apply_changeset_callback (r, "?-- %s\n", no_dot (mod_loc));
+          escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, mod_loc);
+          invoke_apply_changeset_callback (r, "?-- %s\n", no_dot (escaped_tmp));
           rel_add_records (&r->missing_file_for_meta_patch, rel_make_record (mod_loc, id, 0), 0);
           save_patch_for_missing_file (missing_patch_dir, orig_patch_path, mod_loc);
           save_patch_for_missing_file (missing_patch_dir, mod_patch_path, mod_loc);
+          lim_free (0, escaped_tmp);
           continue;
         }
 
@@ -1437,9 +1522,11 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
           safe_close (in_fd);
           safe_close (out_fd);
 
-          invoke_apply_changeset_callback (r, "C-- %s\n", no_dot (target_loc));
+          escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, target_loc);
+          invoke_apply_changeset_callback (r, "C-- %s\n", no_dot (escaped_tmp));
           rel_add_records (&r->metadata_conflict_files, rel_make_record (target_loc, id, 0), 0);
 
+          lim_free (0, escaped_tmp);
           lim_free (0, rej_name);
         }
       else
@@ -1465,8 +1552,10 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
               lim_free (0, target_path_dir);
               lim_free (0, target_path_tmp);
             }
-          invoke_apply_changeset_callback (r, "--  %s\n", no_dot (target_loc));
+          escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, target_loc);
+          invoke_apply_changeset_callback (r, "--  %s\n", no_dot (escaped_tmp));
           rel_add_records (&r->meta_modified_files, rel_make_record (target_loc, id, 0), 0);
+          lim_free (0, escaped_tmp);
         }
 
       lim_free (0, orig_patch_path);
@@ -1493,6 +1582,7 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
       t_uchar * target_path = 0;
       t_uchar * target_spew_loc = 0;
       t_uchar * target_spew_path = 0;
+      t_uchar * escaped_tmp = 0;
       struct stat target_stat;
 
       if (!reverse)
@@ -1512,10 +1602,12 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
 
       if (!target_loc)
         {
-          invoke_apply_changeset_callback (r, "?-/ %s\n", no_dot (mod_loc));
+          escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, mod_loc);
+          invoke_apply_changeset_callback (r, "?-/ %s\n", no_dot (escaped_tmp));
           rel_add_records (&r->missing_dir_for_meta_patch, rel_make_record (mod_loc, id, 0), 0);
           save_patch_for_missing_file (missing_patch_dir, orig_patch_path, mod_loc);
           save_patch_for_missing_file (missing_patch_dir, mod_patch_path, mod_loc);
+          lim_free (0, escaped_tmp);
           continue;
         }
 
@@ -1539,8 +1631,10 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
             safe_chmod (target_path, mod_perms);
           }
 
-        invoke_apply_changeset_callback (r, "--/ %s\n", no_dot (target_loc));
+        escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, target_loc);
+        invoke_apply_changeset_callback (r, "--/ %s\n", no_dot (escaped_tmp));
         rel_add_records (&r->meta_modified_dirs, rel_make_record (target_loc, id, 0), 0);
+        lim_free (0, escaped_tmp);
       }
 
       lim_free (0, orig_patch_path);
@@ -1564,6 +1658,7 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
       t_uchar * target_tmp = 0;
       t_uchar * archive = 0;
       t_uchar * version = 0;
+      t_uchar * escaped_tmp = 0;
       int out_fd;
       struct stat stat_was;
       mode_t mode;
@@ -1587,9 +1682,11 @@ arch_apply_changeset (struct arch_apply_changeset_report * r,
       safe_close (out_fd);
       safe_rename (target_tmp, target_path);
 
-      invoke_apply_changeset_callback (r, "cl  %s\n", no_dot (target_loc));
+      escaped_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, target_loc);
+      invoke_apply_changeset_callback (r, "cl  %s\n", no_dot (escaped_tmp));
       rel_add_records (&r->modified_files, rel_make_record (target_loc, id, 0), 0);
 
+      lim_free (0, escaped_tmp);
       lim_free (0, target_dir);
       lim_free (0, target_tmp);
       lim_free (0, archive);
