@@ -26,7 +26,8 @@ static void arbdelta_callback (void * vfd, char * fmt, va_list ap);
 void
 arch_arbitrary_delta (int chatter_fd, t_uchar * output_dir, t_uchar * tmp_dir, t_uchar * tree_root,
                       struct arch_archive * orig_arch, t_uchar * orig_archive, t_uchar * orig_revision,
-                      struct arch_archive * mod_arch, t_uchar * mod_archive, t_uchar * mod_revision)
+                      struct arch_archive * mod_arch, t_uchar * mod_archive, t_uchar * mod_revision,
+                      int escape_classes)
 {
   t_uchar * orig = 0;
   t_uchar * mod = 0;
@@ -49,7 +50,7 @@ arch_arbitrary_delta (int chatter_fd, t_uchar * output_dir, t_uchar * tmp_dir, t
     }
 
   arch_read_inode_sig (0, &inode_shortcut, mod, mod_archive, mod_revision);
-  arch_make_changeset (&report, orig, mod, output_dir, arch_unspecified_id_tagging, arch_inventory_unrecognized, 0, inode_shortcut, 0);
+  arch_make_changeset (&report, orig, mod, output_dir, arch_unspecified_id_tagging, arch_inventory_unrecognized, 0, inode_shortcut, 0, escape_classes);
 
 
   lim_free (0, orig);
