@@ -214,6 +214,12 @@ arch_cmd_archive_setup (t_uchar * program_name, int argc, char * argv[])
         archive = arch_parse_package_name (arch_ret_archive, default_archive, wants[x][0]);
         non_archive = arch_parse_package_name (arch_ret_non_archive, 0, wants[x][0]);
 
+        if (arch_is_system_package_name (non_archive))
+          {
+            safe_printfmt (2, "%s: user's can not create system categories, branches or versions\n  illegal spec: %s\n", argv[0], non_archive);
+            exit (2);
+          }
+
         if (tag_op == arch_archive_setup_no_tags)
           rel_add_records (&wants_decomposed, rel_make_record (archive, non_archive, 0), 0);
         else
