@@ -116,15 +116,7 @@ arch_cmd_get_changeset (t_uchar * program_name, int argc, char * argv[])
     t_uchar * output_dir = 0;
     struct arch_archive * arch = 0;
 
-    revision_spec = argv[1];
-
-    if (!arch_valid_package_name (revision_spec, arch_maybe_archive, arch_req_patch_level, 0))
-      {
-        safe_printfmt (2, "%s: invalid revision name (%s)\n",
-                       argv[0], revision_spec);
-        exit (2);
-      }
-
+    revision_spec = arch_fqrvsn_from_tree_and_input (argv[0], argv[1], ".");
     archive = arch_parse_package_name (arch_ret_archive, default_archive, revision_spec);
     revision = arch_parse_package_name (arch_ret_non_archive, 0, revision_spec);
 
