@@ -26,9 +26,10 @@
 #include "tla/libarch/libraries.h"
 #include "tla/libarch/local-cache.h"
 #include "tla/libarch/library-txn.h"
-#include "tla/libarch/cmd-get.h"
+#include "tla/libarch/cmd.h"
 #include "tla/libarch/cmd-library-add.h"
 #include "tla/libarch/cmdutils.h"
+#include "tla/libarch/inode-sig.h"
 
 
 static t_uchar * usage = "[options] revision [dir]";
@@ -70,7 +71,9 @@ static t_uchar * version_string = (cfg__std__package " from regexps.com\n"
   OP (opt_silent, "s", "silent", 0, \
       "no output") \
   OP (opt_no_greedy_add, 0, "no-greedy-add", 0, \
-      "do not allow greedy libraries to add revisions")
+      "do not allow greedy libraries to add revisions")\
+  OP (opt_unescaped, 0, "unescaped", 0, \
+      "show filenames in unescaped form")
 
 
 t_uchar arch_cmd_get_help[] = ("construct a project tree for a revision\n"
@@ -108,6 +111,7 @@ arch_cmd_get (t_uchar * program_name, int argc, char * argv[])
   int library = 0;
   int silent = 0;
   int permit_greed = 1;
+  int escape_classes = arch_escape_classes;
 
   safe_buffer_fd (1, 0, O_WRONLY, 0);
 
@@ -183,11 +187,18 @@ arch_cmd_get (t_uchar * program_name, int argc, char * argv[])
             silent = 1;
             break;
           }
-	case opt_no_greedy_add:
+
+	case opt_unescaped:
 	  {
-	    permit_greed = 0;
+	    escape_classes = 0;
 	    break;
 	  }
+
+ 	case opt_no_greedy_add:
+ 	  {
+ 	    permit_greed = 0;
+ 	    break;
+ 	  }
         }
     }
 
@@ -353,8 +364,8 @@ arch_cmd_get (t_uchar * program_name, int argc, char * argv[])
       {
         safe_printfmt (1, "* ensuring library has %s/%s\n", archive, revision);
         safe_flush (1);
-	
-	arch_library_add (1, 1, arch, revision, selected_library, opt_same_dev_path, sparse);
+
+ 	arch_library_add (1, 1, arch, revision, selected_library, opt_same_dev_path, sparse, escape_classes);
       }
 
     /* make hardlinks or build-revision
@@ -370,6 +381,7 @@ arch_cmd_get (t_uchar * program_name, int argc, char * argv[])
         safe_printfmt (1, "* hard linking to library\n");
         safe_flush (1);
         build_partial_link_tree (arch_library_find (0, archive, revision, 1), tmp_dir, index);
+        arch_snap_inode_sig (tmp_dir, archive, revision);
 
         lim_free (0, revision_dir);
       }
