@@ -293,7 +293,7 @@ arch_grab (int chatter_fd, t_uchar * location)
        * Time to grab the archive
        */
       final_revision = find_latest_revision (archive_name, target_revision);
-      arch_set_archive_location(archive_name, archive_location, 0, 1);
+      arch_set_archive_location(archive_name, archive_location, 0, ARCH_REG_FAIL_QUIET);
 
       archive = arch_archive_connect (archive_name, 0);
 
