@@ -29,6 +29,7 @@
 #include "tla/libarch/pristines.h"
 #include "tla/libarch/build-revision.h"
 #include "tla/libarch/replay.h"
+#include "tla/libarch/cmd.h"
 #include "tla/libarch/cmd-replay.h"
 #include "tla/libarch/cmdutils.h"
 
@@ -72,7 +73,9 @@ static t_uchar * version_string = (cfg__std__package " from regexps.com\n"
       "Instead of modifying the project tree in-place,\n" \
       "make a copy of it to DEST and apply the result to that") \
   OP (opt_skip_present, 0, "skip-present", 0, \
-      "skip patches that contain 1 or more patch logs already in this tree")
+      "skip patches that contain 1 or more patch logs already in this tree") \
+  OP (opt_unescaped, 0, "unescaped", 0, \
+      "show filenames in unescaped form")
 
 
 t_uchar arch_cmd_replay_help[] = ("apply revision changesets to a project tree\n"
@@ -130,6 +133,7 @@ arch_cmd_replay (t_uchar * program_name, int argc, char * argv[])
   int forward = 0;
   int skip_present = 0;
   enum op action = arch_replay_op_unspecified;
+  int escape_classes = arch_escape_classes;
   int exit_status = 0;
 
   vu_push_dash_handler (0);
@@ -208,6 +212,12 @@ arch_cmd_replay (t_uchar * program_name, int argc, char * argv[])
             skip_present = 1;
             break;
           }
+
+	case opt_unescaped:
+	  {
+	    escape_classes = 0;
+	    break;
+	  }
         }
     }
 
@@ -336,7 +346,7 @@ arch_cmd_replay (t_uchar * program_name, int argc, char * argv[])
               t_uchar * revision = arch_parse_package_name (arch_ret_non_archive, 0, argv[i]);
 	      arch_check_for (arch, arch_req_patch_level, revision);
 
-              exit_status = arch_replay_exact (1, 0, dest, arch, revision, reverse, forward);
+              exit_status = arch_replay_exact (1, 0, dest, arch, revision, reverse, forward, escape_classes);
               lim_free (0, revision);
               lim_free (0, archive);
               arch_archive_close (arch);
@@ -403,7 +413,7 @@ arch_cmd_replay (t_uchar * program_name, int argc, char * argv[])
                   lim_free (0, t);
                 }
 
-              exit_status = arch_replay_list (1, 0, dest, arch, revisions, 0, forward);
+              exit_status = arch_replay_list (1, 0, dest, arch, revisions, 0, forward, escape_classes);
 
               lim_free (0, version);
               rel_free_table (revisions);
@@ -437,7 +447,8 @@ arch_cmd_replay (t_uchar * program_name, int argc, char * argv[])
                 }
             }
 
-          exit_status = arch_replay_fqlist (1, 0, dest, default_archive, revisions, reverse, forward);
+          exit_status = arch_replay_fqlist (1, 0, dest, default_archive, revisions,
+                                            reverse, forward, escape_classes);
 
           safe_close (in_fd);
           rel_free_table (revisions);
