@@ -15,6 +15,7 @@
 #include "tla/libarch/project-tree.h"
 #include "tla/libarch/my.h"
 #include "tla/libarch/archive.h"
+#include "tla/libarch/cmdutils.h"
 #include "tla/libarch/cmd-revisions.h"
 
 
@@ -191,6 +192,8 @@ arch_cmd_revisions (t_uchar * program_name, int argc, char * argv[])
     version = arch_parse_package_name (arch_ret_package_version, 0, version_spec);
 
     arch = arch_archive_connect (archive, 0);
+    if (argc == 2)
+      arch_check_for (arch, arch_req_version, version);
     revisions = arch_archive_revisions (arch, version, full);
 
     if (reverse)
