@@ -28,12 +28,12 @@
 
 
 /* __STDC__ prototypes for static functions */
+static void arch_update_inode_sig (rel_table sig, assoc_table newsig, assoc_table id_list);
 static void inode_sig_callback (t_uchar * path, struct stat * stat_buf,
-                                enum arch_inventory_category category,
-                                t_uchar * id, int has_source_name,
-                                void * closure);
-static void 
-arch_update_inode_sig (rel_table sig, assoc_table newsig, assoc_table id_list);
+                    enum arch_inventory_category category,
+                    t_uchar * id, int has_source_name,
+                    void * closure,
+                    int escape_classes);
 static t_uchar * arch_inode_sig_dir (t_uchar * tree_root);
 static t_uchar * arch_inode_sig_file (t_uchar * tree_root, t_uchar * archive, t_uchar * revision);
 static t_uchar * arch_inode_sig_tmp_file (t_uchar * tree_root, t_uchar * archive, t_uchar * revision);
@@ -41,6 +41,7 @@ static int arch_creat_inode_sig_file (t_uchar * tree_root, t_uchar * archive, t_
 static void arch_finish_inode_sig_file (t_uchar * tree_root, t_uchar * archive, t_uchar * revision, int fd);
 static void arch_prune_inode_sig_dir (t_uchar * tree_root, t_uchar * current_inode_sig_file);
 static void prune_old_ctime_cruft (rel_table the_table);
+static void prune_old_dev_cruft (rel_table the_table);
 
 
 
@@ -49,8 +50,7 @@ arch_statb_inode_sig (struct stat * statb)
 {
   int fd = make_output_to_string_fd ();
 
-  safe_printfmt (fd, "dev=%lu:ino=%lu:mtime=%lu:size=%lu",
-                 (t_ulong)statb->st_dev,
+  safe_printfmt (fd, "ino=%lu:mtime=%lu:size=%lu",
                  (t_ulong)statb->st_ino,
                  (t_ulong)statb->st_mtime,
                  (t_ulong)statb->st_size);
@@ -75,7 +75,7 @@ arch_tree_inode_sig (t_uchar * tree_root)
 
   arch_get_inventory_naming_conventions (&options, ".");
 
-  arch_inventory_traversal (&options, ".", inode_sig_callback, (void *)&answer);
+  arch_inventory_traversal (&options, ".", inode_sig_callback, (void *)&answer, 0);
 
   arch_free_inventory_naming_conventions (&options);
 
@@ -202,6 +202,7 @@ arch_read_inode_sig (rel_table * as_table, assoc_table * as_assoc,
       the_table = rel_read_table (fd, 2, "arch", sig_file);
 
       prune_old_ctime_cruft (the_table);
+      prune_old_dev_cruft (the_table);
 
       if (as_assoc)
         *as_assoc = rel_to_assoc (the_table, 0, 1);
@@ -293,7 +294,8 @@ static void
 inode_sig_callback (t_uchar * path, struct stat * stat_buf,
                     enum arch_inventory_category category,
                     t_uchar * id, int has_source_name,
-                    void * closure)
+                    void * closure,
+                    int escape_classes)
 {
   rel_table * answer = (rel_table *)closure;
   t_uchar * signature = arch_statb_inode_sig (stat_buf);
@@ -459,7 +461,7 @@ prune_old_ctime_cruft (rel_table the_table)
       if (!third_colon)
         return;
 
-      if (str_cmp_n (third_colon + 1, MIN (5, str_length (third_colon + 1)), "ctime", 5))
+      if (str_cmp_n (third_colon + 1, MIN (5, str_length (third_colon + 1)), "ctime", (size_t)5))
         return;
 
       fourth_colon = str_chr_index (third_colon + 1, ':');
@@ -471,6 +473,31 @@ prune_old_ctime_cruft (rel_table the_table)
 }
 
 
+static void
+prune_old_dev_cruft (rel_table the_table)
+{
+  int x;
+
+#undef MIN
+#define MIN(A,B) (((A) < (B)) ? (A) : (B))
+
+  for (x = 0; x < rel_n_records (the_table); ++x)
+    {
+      t_uchar * sig = the_table[x][1];
+      t_uchar * first_colon;
+
+      if (str_cmp_n (sig, MIN (3, str_length (sig)), "dev", (size_t)3))
+        return;
+
+      first_colon = str_chr_index (sig, ':');
+      if (!first_colon)
+        return;
+
+      mem_move (sig, first_colon + 1, str_length (first_colon));
+    }
+}
+
+
 
 
 
