@@ -275,7 +275,7 @@ arch_default_id_tagging_method_contents (enum arch_id_tagging_method method)
                                 "#          may be of the form \"<PUNCT> <BASENAME> - <STRING>\", where\n"
                                 "#          <BASENAME> is the basename of the file.   This method\n"
                                 "#          is not recommended, but is retained for backwards\n"
-                                "#          compatability.\n"
+                                "#          compatibility.\n"
                                 "#\n"
                                 "\n"),
                                ("explicit\n"
@@ -911,9 +911,9 @@ arch_set_dont_care_explicit_default (t_uchar * dir)
 
 /*(c file_id)
  * static t_uchar * file_id (int * errn,
- *                              struct alloc_limits * limits,
- *                              enum ftag_method method,
- *                              t_uchar * path);
+ *                           struct alloc_limits * limits,
+ *                           enum ftag_method method,
+ *                           t_uchar * path);
  *
  * Return a newly allocated string containing the inventory id
  * of the file `path' using inventory method `method'.
@@ -924,52 +924,37 @@ arch_set_dont_care_explicit_default (t_uchar * dir)
  */
 static t_uchar *
 file_id (int * errn,
-          struct alloc_limits * limits,
-          enum ftag_method method,
-          int untagged_is_source,
-          t_uchar * path,
-          assoc_table id_tagging_shortcut,
-          struct stat * known_lstat,
-          assoc_table * explicit_skips)
+         struct alloc_limits * limits,
+         enum ftag_method method,
+         int untagged_is_source,
+         t_uchar * path,
+         assoc_table id_tagging_shortcut,
+         struct stat * known_lstat,
+         assoc_table * explicit_skips)
 {
-  t_uchar * answer;
-  t_uchar * as_file;
-  t_uchar * basename;
-  t_uchar * dir;
-  t_uchar * dir_as_file;
-  t_uchar * dir_basename;
-  t_uchar * id_file;
+  t_uchar * answer = 0;
+  t_uchar * as_file = 0;
+  t_uchar * basename = 0;
+  t_uchar * dir = 0;
+  t_uchar * dir_as_file = 0;
+  t_uchar * dir_basename = 0;
+  t_uchar * id_file = 0;
   struct stat stat_buf;
   int is_dir;
   int is_symlink;
 
-
-  answer = 0;
-  as_file = 0;
-  basename = 0;
-  dir = 0;
-  dir_as_file = 0;
-  dir_basename = 0;
-  id_file = 0;
-
   if (!path)
     {
       *errn = EINVAL;
 
     return_answer:
 
-      if (as_file)
-        lim_free (limits, as_file);
-      if (basename)
-        lim_free (limits, basename);
-      if (dir)
-        lim_free (limits, dir);
-      if (dir_as_file)
-        lim_free (limits, dir_as_file);
-      if (dir_basename)
-        lim_free (limits, dir_basename);
-      if (id_file)
-        lim_free (limits, id_file);
+      lim_free (limits, as_file);
+      lim_free (limits, basename);
+      lim_free (limits, dir);
+      lim_free (limits, dir_as_file);
+      lim_free (limits, dir_basename);
+      lim_free (limits, id_file);
 
       return answer;
     }
@@ -1183,6 +1168,12 @@ file_id (int * errn,
 
 
 
+t_uchar *
+arch_id_from_explicit_file (int *errn, t_uchar * path)
+{
+  return explicit_id (errn, 0, 0, 0, path, "x_", 0);
+}
+
 static int
 is_at_or_underneath_archdir (char * rel_file)
 {
@@ -1470,7 +1461,15 @@ implicit_id (int * errn,
 
               eol = str_chr_index_n (buf + line, amt - line, '\n');
               if (!eol)
-                eol = buf + amt;
+                /* end of buffer no eol */
+                {
+                  if (bottom)
+                    eol = buf + amt;
+                  else
+                    {
+                      panic("Top-of-file arch tag crosses 1k boundary");
+                    }
+                }
 
               if (0 == (eol - (t_uchar *)(buf + line)))
                 {
