@@ -119,6 +119,7 @@ arch_nearest_star_ancester (t_uchar * from_tree_root, t_uchar * from_archive, t_
   rel_free_table (from_logs_for_from_version);
   rel_free_table (common_from_logs);
   lim_free (0, latest_common_from_revision);
+  rel_free_table (from_merges_from_to);
   lim_free (0, highest_common_to_revision);
   lim_free (0, from_rev_of_highest_common_to_revision);
 
@@ -131,7 +132,8 @@ int
 arch_star_merge (int chatter_fd,
                  struct arch_archive * from_arch, t_uchar * from_archive, t_uchar * from_revision,
                  t_uchar * to_tree_root, struct arch_archive * to_arch, t_uchar * to_archive, t_uchar * to_version,
-                 t_uchar * cache_dir, t_uchar * changeset_output, int use_diff3, int forward)
+                 t_uchar * cache_dir, t_uchar * changeset_output, int use_diff3, int forward,
+                 int escape_classes)
 {
   int answer = 2;
   t_uchar * tmp_dir = 0;
@@ -180,7 +182,7 @@ arch_star_merge (int chatter_fd,
         }
 
       arch_read_inode_sig (0, &inode_shortcut, from_tree_root, orig_archive, orig_revision);
-      arch_make_changeset (&make_report, orig_tree_root, from_tree_root, changeset, arch_unspecified_id_tagging, arch_inventory_unrecognized, 0, inode_shortcut, 0);
+      arch_make_changeset (&make_report, orig_tree_root, from_tree_root, changeset, arch_unspecified_id_tagging, arch_inventory_unrecognized, 0, inode_shortcut, 0, escape_classes);
 
       if (changeset_output)
         answer = 0;
@@ -201,12 +203,15 @@ arch_star_merge (int chatter_fd,
           arch_chatter (chatter_fd, "* applying changeset\n");
           safe_flush (chatter_fd);
 
-          arch_apply_changeset (&apply_report, changeset, to_tree_root, arch_unspecified_id_tagging, arch_inventory_unrecognized, 0, forward, use_diff3, older_files_table, yours_files_table);
+          arch_apply_changeset (&apply_report, changeset, to_tree_root, arch_unspecified_id_tagging, arch_inventory_unrecognized, 0, forward, use_diff3, older_files_table, yours_files_table, escape_classes);
 
           if (arch_conflicts_occured (&apply_report))
             answer = 1;
           else
             answer = 0;
+
+          free_assoc_table (older_files_table);
+          free_assoc_table (yours_files_table);
         }
 
       arch_free_make_changeset_report_data (&make_report);
